@@ -14,10 +14,16 @@
 #    under the License.
 
 """
-Simple class that stores security context information in the web request.
+Base class for holding contextual information of a request
+
+This class has several uses:
+
+* Used for storing security information in a web request.
+* Used for passing contextual details to oslo.log.
 
 Projects should subclass this class if they wish to enhance the request
-context or provide additional information in their specific WSGI pipeline.
+context or provide additional information in their specific WSGI pipeline
+or logging context.
 """
 
 import itertools
@@ -29,7 +35,8 @@ _request_store = threading.local()
 
 
 def generate_request_id():
-    return b'req-' + str(uuid.uuid4()).encode('ascii')
+    """Generate a unique request id."""
+    return 'req-%s' % uuid.uuid4()
 
 
 class RequestContext(object):
@@ -68,9 +75,11 @@ class RequestContext(object):
             self.update_store()
 
     def update_store(self):
+        """Store the context in the current thread."""
         _request_store.context = self
 
     def to_dict(self):
+        """Return a dictionary of context attributes."""
         user_idt = (
             self.user_idt_format.format(user=self.user or '-',
                                         tenant=self.tenant or '-',
@@ -91,8 +100,14 @@ class RequestContext(object):
                 'resource_uuid': self.resource_uuid,
                 'user_identity': user_idt}
 
+    def get_logging_values(self):
+        """Return a dictionary of logging specific context attributes."""
+        values = self.to_dict()
+        return values
+
     @classmethod
     def from_dict(cls, ctx):
+        """Construct a context object from a provided dictionary."""
         return cls(
             auth_token=ctx.get("auth_token"),
             user=ctx.get("user"),
@@ -106,6 +121,28 @@ class RequestContext(object):
             request_id=ctx.get("request_id"),
             resource_uuid=ctx.get("resource_uuid"))
 
+    @classmethod
+    def from_environ(cls, environ, **kwargs):
+        """Load a context object from a request environment.
+
+        If keyword arguments are provided then they override the values in the
+        request environment.
+
+        :param environ: The environment dictionary associated with a request.
+        :type environ: dict
+        """
+        # Load a new context object from the environment variables set by
+        # auth_token middleware. See:
+        # http://docs.openstack.org/developer/keystonemiddleware/api/keystonemiddleware.auth_token.html#what-auth-token-adds-to-the-request-for-use-by-the-openstack-service
+        kwargs.setdefault('auth_token', environ.get('HTTP_X_AUTH_TOKEN'))
+        kwargs.setdefault('user', environ.get('HTTP_X_USER_ID'))
+        kwargs.setdefault('tenant', environ.get('HTTP_X_PROJECT_ID'))
+        kwargs.setdefault('user_domain', environ.get('HTTP_X_USER_DOMAIN_ID'))
+        kwargs.setdefault('project_domain',
+                          environ.get('HTTP_X_PROJECT_DOMAIN_ID'))
+
+        return cls(**kwargs)
+
 
 def get_admin_context(show_deleted=False):
     """Create an administrator context."""
@@ -133,7 +170,7 @@ def get_context_from_function_and_args(function, args, kwargs):
 
 def is_user_context(context):
     """Indicates if the request context is a normal user."""
-    if not context:
+    if not context or not isinstance(context, RequestContext):
         return False
     if context.is_admin:
         return False
